using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using Microsoft.Extensions.Logging;

namespace FitConnect;

public class EncryptionBaseClass {
    private readonly ILogger _logger;
    private readonly RSA _rsa;
    private RSA? _publicKey;
    private RSA? _privateKey;

    protected EncryptionBaseClass(ILogger logger) {
        _logger = logger;
        _rsa = RSA.Create(2048);
    }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="cert"></param>
    /// <returns></returns>
    private bool CheckCertificate(X509Certificate2 cert) =>
        cert.HasPrivateKey; //  && cert.Verify();


    public void ImportCertificate(X509Certificate2 cert) {
        if (!CheckCertificate(cert)) {
            throw new Exception("Invalid certificate");
        }

        _publicKey = cert.GetRSAPublicKey();
        // _publicKey = RSA.Create(2048);
        // _publicKey.ImportRSAPublicKey(cert.GetPublicKey(), out int _);

        if ((_publicKey?.KeySize ?? 0) == 0)
            throw new Exception("Invalid certificate, no public key");

        _logger.LogInformation("Public key imported {}",
            Convert.ToBase64String(_publicKey.ExportRSAPrivateKey()));

        if (cert.HasPrivateKey) {
            _privateKey = cert.GetRSAPrivateKey();
        }

        if (_privateKey != null)
            _logger.LogInformation("Certificate with private key imported");
        else
            _logger.LogInformation("Certificate has no private key");
    }

    /// <summary>
    /// Import a public key from a PEM file
    /// </summary>
    /// <param name="certificatePath"></param>
    /// <param name="password">Password for the certificate</param>
    /// <exception cref="ArgumentException"></exception>
    /// <exception cref="Exception"></exception>
    public void ImportCertificate(string certificatePath, string password) {
        _logger.LogInformation("Importing certificate {CertPath}", certificatePath);
        var cert =
            new X509Certificate2(certificatePath, password, X509KeyStorageFlags.MachineKeySet);

        if (!CheckCertificate(cert)) {
            throw new ArgumentException("Certificate is not valid");
        }

        var parameters = cert.PublicKey.GetRSAPublicKey()?.ExportParameters(true);
        if (parameters == null) {
            throw new Exception("Could not get public key from certificate");
        }

        _rsa.ImportParameters(parameters.Value);
    }

    public byte[] DecryptDataAsync(byte[] data) {
        return (_privateKey ?? _rsa).Decrypt(data, RSAEncryptionPadding.OaepSHA256);
    }

    public byte[] ExportPublicKey() {
        return _rsa.ExportRSAPublicKey();
    }

    public byte[] ExportPrivateKey() {
        return _rsa.ExportRSAPrivateKey();
    }

    public byte[] EncryptData(byte[] data) {
        return (_publicKey ?? _rsa).Encrypt(data, RSAEncryptionPadding.OaepSHA256);
    }

    public byte[] EncryptData(byte[] data, byte[] publicKey) {
        _logger?.LogInformation(
            "Encrypting data with public key: {}",
            Convert.ToBase64String(_rsa.ExportRSAPublicKey()));

        _rsa.ImportRSAPublicKey(publicKey, out var read);
        return _rsa.Encrypt(data, RSAEncryptionPadding.OaepSHA256);
    }
}
